/**
 * Returns a seeded PRNG function
 * @param {number} seed Seed
 * @returns {() => number}
 */
declare function seedFunc(seed: number): () => number;
//# sourceMappingURL=seed-func.d.ts.map

/**
 * Returns a PRNG function that returns unique values from the provided range.
 * @param {number} min Minimum integer
 * @param {number} max Maximum integer
 * @param {() => number} engine Engine to be used as PRNG
 * @returns {() => number}
 */
declare function uniqFuncIntRange(min: number, max: number, engine?: () => number): () => number;
//# sourceMappingURL=uniq-func-int-range.d.ts.map

/**
 * Returns a PRNG function that returns unique values from the provided array.
 * @param {*[]} arr Array of values to pull from
 * @param {() => number} engine Engine to be used as PRNG
 * @returns {() => *}
 */
declare function uniqFuncSequence(arr: any[], engine?: () => number): () => any;
//# sourceMappingURL=uniq-func-sequence.d.ts.map

/**
 * Returns a random boolean (true or false)
 * @param {() => number} engine PRNG of choice
 * @returns {boolean}
 */
declare function bool(engine?: () => number): boolean;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {() => boolean}
 */
declare function boolWithEngine(engine?: () => number): () => boolean;

/**
 * Returns a random character from the provided array
 * @param {string} str
 * @param {() => number} engine PRNG of choice
 * @returns {string}
 */
declare function char(str: string, engine?: () => number): string;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {(str: string) => string}
 */
declare function charWithEngine(engine?: () => number): (str: string) => string;

/**
 * Returns a random value that follows a custom distribution
 * @param {(num: number) => *} func Distribution function that accepts a random number between 0 and 1
 * @param {() => number} engine PRNG of choice
 * @returns {*}
 */
declare function customDist(func: (num: number) => any, engine?: () => number): any;
/**
 * @param {() => number} engine Distribution function that accepts a random number between 0 and 1
 * @returns {(func: (num: number) => *) => *}
 */
declare function customDistWithEngine(engine?: () => number): (func: (num: number) => any) => any;

/**
 * Returns a random value that follows an exponential distribution
 * @param {number} lambda Lambda
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
declare function exponentialDist(lambda: number, engine?: () => number): number;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {(lambda: number) => number}
 */
declare function exponentialDistWithEngine(engine?: () => number): (lambda: number) => number;

/**
 * Returns a random number within the float range
 * @param {number} min Minimum value
 * @param {number} max Maximum value
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
declare function floatRange(min: number, max: number, engine?: () => number): number;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {(min: number, max: number) => number}
 */
declare function floatRangeWithEngine(engine?: () => number): (min: number, max: number) => number;

/**
 * Returns a random number within the integer range
 * @param {number} min Minimum integer
 * @param {number} max Maximum integer
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
declare function intRange(min: number, max: number, engine?: () => number): number;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {(min: number, max: number) => number}
 */
declare function intRangeWithEngine(engine?: () => number): (min: number, max: number) => number;

/**
 * Returns an array with every value in the range in random order
 * @param {number} min Minimum value
 * @param {number} max Maximum value
 * @param {() => number} engine PRNG of choice
 * @returns {number[]}
 */
declare function intSequence(min: number, max: number, engine?: () => number): number[];
/**
 * @param {() => number} engine PRNG of choice
 * @returns {(min: number, max: number) => number[]}
 */
declare function intSequenceWithEngine(engine?: () => number): (min: number, max: number) => number[];

/**
 * Returns a random value that follows a normal distribution
 * @param {number} mean Mean
 * @param {number} stdDev Standard Deviation
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
declare function normalDist(mean: number, stdDev: number, engine?: () => number): number;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {(mean: number, stdDev: number) => number}
 */
declare function normalDistWithEngine(engine?: () => number): (mean: number, stdDev: number) => number;

/**
 * Returns a random number between -1 and 1
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
declare function normalFloat(engine?: () => number): number;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {() => number}
 */
declare function normalFloatWithEngine(engine?: () => number): () => number;

/**
 * Returns a random value from the provided array
 * @param {*[]} arr Array of values
 * @param {() => number} engine PRNG of choice
 * @returns {*}
 */
declare function oneOf(arr: any[], engine?: () => number): any;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {(arr: *[]) => *}
 */
declare function oneOfWithEngine(engine?: () => number): (arr: any[]) => any;

/**
 * Returns a new array with randomized order
 * @param {*[]} arr Array of values
 * @param {() => number} engine PRNG of choice
 * @returns {*[]}
 */
declare function sequence(arr: any[], engine?: () => number): any[];
/**
 * @param {() => number} engine PRNG of choice
 * @returns {(arr: *[]) => *[]}
 */
declare function sequenceWithEngine(engine?: () => number): (arr: any[]) => any[];

/**
 * Returns a random sign (-1 or 1)
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
declare function sign(engine?: () => number): number;
/**
 * @param {() => number} engine PRNG of choice
 * @returns {() => number}
 */
declare function signWithEngine(engine?: () => number): () => number;

/**
 * Returns a valid RFC4122 version4 ID hex string, using the provided engine
 * @param {() => number} engine PRNG of choice
 * @returns {string}
 */
declare function uuid(engine?: () => number): string;
/**
 *
 * @param {() => number} engine PRNG of choice
 * @returns {() => string}
 */
declare function uuidWithEngine(engine?: () => number): () => string;

/**
 * Returns a random value from the provided array, biased towards provided weights
 * @param {number[]} nums Array of numbers (integer or float)
 * @param {number[]} weights Array of weights (integer or float)
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
declare function weighted(nums: number[], weights: number[], engine?: () => number): number;
/**
 *
 * @param {() => number} engine PRNG of choice
 * @returns {(nums: number[], weights: number[]) => number}
 */
declare function weightWithEngine(engine?: () => number): (nums: number[], weights: number[]) => number;

export { bool, boolWithEngine, char, charWithEngine, customDist, customDistWithEngine, exponentialDist, exponentialDistWithEngine, floatRange, floatRangeWithEngine, intRange, intRangeWithEngine, intSequence, intSequenceWithEngine, normalDist, normalDistWithEngine, normalFloat, normalFloatWithEngine, oneOf, oneOfWithEngine, seedFunc, sequence, sequenceWithEngine, sign, signWithEngine, uniqFuncIntRange, uniqFuncSequence, uuid, uuidWithEngine, weightWithEngine, weighted };
