/**
 * The default engine for Aimless.js, utilizing crypto when available
 * @returns {number} a number >= 0 and < 1
 */
var defaultEngine = function defaultEngine() {
  try {
    // Credit @ Tamás Sallai
    var crypto = window.crypto || window.msCrypto;
    var buffer = new ArrayBuffer(8);
    var ints = new Int8Array(buffer);
    crypto.getRandomValues(ints);
    ints[7] = 63;
    ints[6] |= 0xf0;
    return new DataView(buffer).getFloat64(0, true) - 1;
  } catch (e) {
    return Math.random();
  }
};

/**
 * Generates an array containing integers between min and max
 * @param {number} min
 * @param {number} max
 * @returns {number[]}
 */
var generateListFromRange = function generateListFromRange(min, max) {
  var result = [];
  for (var i = min; i <= max; i++) {
    result.push(i);
  }
  return result;
};

/**
 * Generates a random integer within range >= i <= j
 * @param {number} i min
 * @param {number} j max
 * @param {() => number} engine 
 * @returns {number}
 */
var randIntRange = function randIntRange(i, j) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  var min = Math.ceil(i);
  var max = Math.floor(j);
  return Math.floor(engine() * (max - min + 1)) + min;
};

/**
 * Slices an item at index i out of the array arr
 * @param {*[]} arr array
 * @param {number} i index
 * @returns {*[]}
 */
var sliceOut = function sliceOut(arr, i) {
  return arr.slice(0, i).concat(arr.slice(i + 1));
};

/**
 * Returns a random number within the integer range
 * @param {number} min Minimum integer
 * @param {number} max Maximum integer
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
var intRange = function intRange(min, max) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  return randIntRange(min, max, engine);
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {(min: number, max: number) => number}
 */
var intRangeWithEngine = function intRangeWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (min, max) {
    return intRange(min, max, engine);
  };
};

/**
 * Returns a random boolean (true or false)
 * @param {() => number} engine PRNG of choice
 * @returns {boolean}
 */
var bool = function bool() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return !!intRange(0, 1, engine);
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {() => boolean}
 */
var boolWithEngine = function boolWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function () {
    return bool(engine);
  };
};

/**
 * Returns a random character from the provided array
 * @param {string} str
 * @param {() => number} engine PRNG of choice
 * @returns {string}
 */
var char = function char(str) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  return str[intRange(0, str.length - 1, engine)];
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {(str: string) => string}
 */
var charWithEngine = function charWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (str) {
    return char(str, engine);
  };
};

/**
 * Returns a random value that follows a custom distribution
 * @param {(num: number) => *} func Distribution function that accepts a random number between 0 and 1
 * @param {() => number} engine PRNG of choice
 * @returns {*}
 */
var customDist = function customDist(func) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  return func(engine());
};

/**
 * @param {() => number} engine Distribution function that accepts a random number between 0 and 1
 * @returns {(func: (num: number) => *) => *}
 */
var customDistWithEngine = function customDistWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (func) {
    return customDist(func, engine);
  };
};

/**
 * Returns a random value that follows an exponential distribution
 * @param {number} lambda Lambda
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
var exponentialDist = function exponentialDist(lambda) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  return -Math.log(1 - engine()) / lambda;
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {(lambda: number) => number}
 */
var exponentialDistWithEngine = function exponentialDistWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (lambda) {
    return exponentialDist(lambda, engine);
  };
};

/**
 * Returns a random number within the float range
 * @param {number} min Minimum value
 * @param {number} max Maximum value
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
var floatRange = function floatRange(min, max) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  return engine() * (max - min) + min;
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {(min: number, max: number) => number}
 */
var floatRangeWithEngine = function floatRangeWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (min, max) {
    return floatRange(min, max, engine);
  };
};

/**
 * Returns a new array with randomized order
 * @param {*[]} arr Array of values
 * @param {() => number} engine PRNG of choice
 * @returns {*[]}
 */
var sequence = function sequence(arr) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  var result = [];
  var tempArr = arr;
  var i;
  while (result.length < arr.length) {
    i = randIntRange(0, tempArr.length - 1, engine);
    result.push(tempArr[i]);
    tempArr = sliceOut(tempArr, i);
  }
  return result;
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {(arr: *[]) => *[]}
 */
var sequenceWithEngine = function sequenceWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (arr) {
    return sequence(arr, engine);
  };
};

/**
 * Returns an array with every value in the range in random order
 * @param {number} min Minimum value
 * @param {number} max Maximum value
 * @param {() => number} engine PRNG of choice
 * @returns {number[]}
 */
var intSequence = function intSequence(min, max) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  return sequence(generateListFromRange(min, max), engine);
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {(min: number, max: number) => number[]}
 */
var intSequenceWithEngine = function intSequenceWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (min, max) {
    return intSequence(min, max, engine);
  };
};

/**
 * Returns a random value that follows a normal distribution
 * @param {number} mean Mean
 * @param {number} stdDev Standard Deviation
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
var normalDist = function normalDist(mean, stdDev) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  // Box-Muller transform
  var u, v, s;
  do {
    u = engine() * 2 - 1;
    v = engine() * 2 - 1;
    s = Math.pow(u, 2) + Math.pow(v, 2);
  } while (s === 0 || s >= 1);
  var t = Math.sqrt(-2 * Math.log(s) / s);
  var rand = mean + stdDev * u * t;
  return rand;
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {(mean: number, stdDev: number) => number}
 */
var normalDistWithEngine = function normalDistWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (mean, stdDev) {
    return normalDist(mean, stdDev, engine);
  };
};

/**
 * Returns a random number between -1 and 1
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
var normalFloat = function normalFloat() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return floatRange(-1, 1, engine);
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {() => number}
 */
var normalFloatWithEngine = function normalFloatWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function () {
    return normalFloat(engine);
  };
};

/**
 * Returns a random value from the provided array
 * @param {*[]} arr Array of values
 * @param {() => number} engine PRNG of choice
 * @returns {*}
 */
var oneOf = function oneOf(arr) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  return arr[intRange(0, arr.length - 1, engine)];
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {(arr: *[]) => *}
 */
var oneOfWithEngine = function oneOfWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (arr) {
    return oneOf(arr, engine);
  };
};

/**
 * Returns a seeded PRNG function
 * @param {number} seed Seed
 * @returns {() => number}
 */
var seedFunc = function seedFunc(seed) {
  // Park-Miller PRNG
  var currentSeed = seed % 2147483647;
  return function () {
    currentSeed = currentSeed * 16807 % 2147483647;
    return (currentSeed - 1) / 2147483646;
  };
};

/**
 * Returns a random sign (-1 or 1)
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
var sign = function sign() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return bool(engine) ? 1 : -1;
};

/**
 * @param {() => number} engine PRNG of choice
 * @returns {() => number}
 */
var signWithEngine = function signWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function () {
    return sign(engine);
  };
};

/**
 * Returns a PRNG function that returns unique values from the provided array.
 * @param {*[]} arr Array of values to pull from
 * @param {() => number} engine Engine to be used as PRNG
 * @returns {() => *}
 */
var uniqFuncSequence = function uniqFuncSequence(arr) {
  var engine = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultEngine;
  var tempArr = arr;
  return function () {
    if (!tempArr.length) return null;
    var i = randIntRange(0, tempArr.length - 1, engine);
    var result = tempArr[i];
    tempArr = sliceOut(tempArr, i);
    return result;
  };
};

/**
 * Returns a PRNG function that returns unique values from the provided range.
 * @param {number} min Minimum integer
 * @param {number} max Maximum integer
 * @param {() => number} engine Engine to be used as PRNG
 * @returns {() => number}
 */
var uniqFuncIntRange = function uniqFuncIntRange(min, max) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  return uniqFuncSequence(generateListFromRange(min, max), engine);
};

/**
 * Returns a valid RFC4122 version4 ID hex string, using the provided engine
 * @param {() => number} engine PRNG of choice
 * @returns {string}
 */
var uuid = function uuid() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  // Credit @Alexey Silin from https://gist.github.com/1308368
  /** @type {*} */
  var a = '';
  /** @type {*} */
  var b = '';
  for (b = a = ''; a++ < 36; b += ~a % 5 | a * 3 & 4 ? (a ^ 15 ? 8 ^ engine() * (a ^ 20 ? 16 : 4) : 4).toString(16) : '-') {}
  return b;
};

/**
 * 
 * @param {() => number} engine PRNG of choice
 * @returns {() => string}
 */
var uuidWithEngine = function uuidWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function () {
    return uuid(engine);
  };
};

/**
 * Returns a random value from the provided array, biased towards provided weights
 * @param {number[]} nums Array of numbers (integer or float)
 * @param {number[]} weights Array of weights (integer or float)
 * @param {() => number} engine PRNG of choice
 * @returns {number}
 */
var weighted = function weighted(nums, weights) {
  var engine = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultEngine;
  if (nums.length !== weights.length) {
    throw new Error('Every provided number must have a corresponding weight.');
  }
  var totalWeight = weights.reduce(function (sum, weight) {
    return sum + weight;
  }, 0);
  var rand = engine() * totalWeight;
  var cumulativeWeight = 0;
  var selectedIndex = 0;
  for (var i = 0; i < nums.length; i++) {
    cumulativeWeight += weights[i];
    if (rand < cumulativeWeight) {
      selectedIndex = i;
      break;
    }
  }
  return nums[selectedIndex];
};

/**
 * 
 * @param {() => number} engine PRNG of choice
 * @returns {(nums: number[], weights: number[]) => number}
 */
var weightWithEngine = function weightWithEngine() {
  var engine = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultEngine;
  return function (nums, weights) {
    return weighted(nums, weights, engine);
  };
};

export { bool, boolWithEngine, char, charWithEngine, customDist, customDistWithEngine, exponentialDist, exponentialDistWithEngine, floatRange, floatRangeWithEngine, intRange, intRangeWithEngine, intSequence, intSequenceWithEngine, normalDist, normalDistWithEngine, normalFloat, normalFloatWithEngine, oneOf, oneOfWithEngine, seedFunc, sequence, sequenceWithEngine, sign, signWithEngine, uniqFuncIntRange, uniqFuncSequence, uuid, uuidWithEngine, weightWithEngine, weighted };
